import 'package:flutter/material.dart';

/// Tipos de animaciones
enum AnimationType {
  normal,
  fadeIn,
}

/// Main class, [context] es el BuildContext de la aplcación en este momento
/// [child] es el widget a navegar, [animation] es el tipo de animación
class RouteTransition {
  final BuildContext context;
  final Widget child;
  final AnimationType animation;
  final Duration duration;
  final bool replacement;

  RouteTransition({
    @required this.context,
    @required this.child,
    this.animation = AnimationType.normal,
    this.duration = const Duration(milliseconds: 300),
    this.replacement = false,
  }) {
    switch (this.animation) {
      case AnimationType.normal:
        this._normalTransition();
        break;
      case AnimationType.fadeIn:
        this._fadeInTransition();
        break;
    }
  }

  /// controlador de Transición normal
  void _normalTransition() {
    final route = MaterialPageRoute(builder: (_) => this.child);

    if (this.replacement) {
      this._pushReplacementPage(route);
    } else {
      this._pushPage(route);
    }
  }

  /// controlador de transición con fadein
  void _fadeInTransition() {
    final route = PageRouteBuilder(
      pageBuilder: (_, __, ___) => this.child,
      transitionDuration: this.duration,
      transitionsBuilder: (_, animation, __, child) {
        return FadeTransition(
          child: this.child,
          opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
            CurvedAnimation(parent: animation, curve: Curves.easeOut),
          ),
        );
      },
    );

    if (this.replacement) {
      this._pushReplacementPage(route);
    } else {
      this._pushPage(route);
    }
  }

  /// Push normal de la página
  void _pushPage(Route route) => Navigator.push(context, route);

  /// Push replacement normal de la página
  void _pushReplacementPage(Route route) =>
      Navigator.pushReplacement(context, route);
}
